#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#define SIMONMAX 4
#define WHACKMAX 3

#define NOTE_C4  262
#define NOTE_G3  196
#define NOTE_A3  220
#define NOTE_B3  247
#define NOTE_C4  262
#define NOTE_G2  98

int melody[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
};

int noteDurations[] = {
  4, 8, 8, 4, 4, 4, 4, 4
};

//definition of leds
#define LED_V 6
#define LED_A 7
#define LED_R 9
#define LED_G 10

//definition of buttons
#define BUTTON_G 2
#define BUTTON_V 3
#define BUTTON_A 4
#define BUTTON_R 5


#define BUZZER 8

int buttonStateR = 0;
int buttonStateV = 0;
int buttonStateA = 0;
int buttonStateG = 0;

int game_choosed = 0;
int game_started = 0;
int lvl_simon = 0;
int lvl_whack = 0;

int simon_score = 0;
int whack_score = 0;
int close_simon = 0;
int close_whack = 0;

int ts=0;

//initializați un obiect "lcd", care are ca parametri
// adresa: 0x27 sau 0x3F
// numărul de caractere pe o linie: 16
// numărul de linii : 2
LiquidCrystal_I2C lcd(0x27,16,2);  

int leds[4] = {LED_R, LED_A, LED_V, LED_G};
int buttons[4] = {BUTTON_R, BUTTON_A, BUTTON_V, BUTTON_G};

int s_lvl = 1;
int w_lvl = 1;
int random_leds[10];




void setup()
{
   Serial.begin(9600);

   pinMode(LED_R, OUTPUT);
   pinMode(LED_A, OUTPUT);
   pinMode(LED_V, OUTPUT);
   pinMode(LED_G, OUTPUT);
   
   digitalWrite(LED_R, LOW);
   digitalWrite(LED_A, LOW);
   digitalWrite(LED_V, LOW);
   digitalWrite(LED_G, LOW);
   
   pinMode(BUTTON_R, INPUT);
   pinMode(BUTTON_A, INPUT);
   pinMode(BUTTON_V, INPUT);
   pinMode(BUTTON_G, INPUT);
   
   lcd.init();                      
   lcd.backlight();
   lcd.clear();

   for(int i = 0;i < 4; i++){

     digitalWrite(leds[i], HIGH);
     delay(300);
     digitalWrite(leds[i], LOW);
   }
   start_song();
 
}


void win_song(){
  lcd.clear();
  lcd.print("Correct!");
  
      tone(BUZZER,  NOTE_C4 , 400);
      tone(BUZZER,  NOTE_G3 , 200);
      for(int j = 0; j < 4; j++){
        digitalWrite(leds[j], HIGH);
        delay(200); 
      }
      
      tone(BUZZER,  NOTE_G3 , 200);
      tone(BUZZER,  NOTE_C4 , 400); 
      for(int j = 3; j >= 0; j--){
        digitalWrite(leds[j], LOW);
        delay(200); 
      }  
      delay(200);
}

void lose_song(){
  lcd.clear();
  lcd.print("WRONG!");
    for(int i = 0; i < 3; i++){
      for(int j = 0; j < 4; j++){
        digitalWrite(leds[j], HIGH);
      }
      tone(BUZZER,  NOTE_G2 , 300);
      delay(200); 
      for(int j = 0; j < 4; j++){
        digitalWrite(leds[j], LOW);
      } 
      delay(200);   
    }
}

void start_song()
{
  for (int thisNote = 0; thisNote < 8; thisNote++) {
    int analogValue = analogRead(POT);
    int pot_value = map(analogValue, 0, 1023, 20, 100);
    int noteDuration = 1000 / noteDurations[thisNote];
    tone(BUZZER,  melody[thisNote] - pot_value , noteDuration);
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    noTone(BUZZER);
  }
}


void play_simon(){
  if(lvl_simon == SIMONMAX){
     close_simon = 1;
     return ;
   }
  
   lcd.clear();
   lcd.print("SIMON - LVL");
   lcd.print(lvl_simon);
   lcd.setCursor(0,1);
   lcd.print("Your score: ");
   lcd.print(simon_score);
   
   for(int i = 0; i < s_lvl; i++){
       random_leds[i] = leds[random(0,3)];
       int analogValue = analogRead(POT);
       int pot_value = map(analogValue, 0, 1023, 20, 100);
       tone(BUZZER,  melody[i]- pot_value, 1000 / noteDurations[i]);
       digitalWrite(random_leds[i], HIGH);
       delay(500);
       noTone(BUZZER);
       digitalWrite(random_leds[i], LOW);
       delay(100);
    }
    wait_for_player(s_lvl);    
  
}

void play_note(int note){
  int analogValue = analogRead(POT);
  int pot_value = map(analogValue, 0, 1023, 20, 100);
  tone(BUZZER,  melody[note]- pot_value, 1000 / noteDurations[note]);
  
}
void wait_for_player(int lvl){
   int leds_player[lvl]; 
   int buttons_pressed = 0;
   for(buttons_pressed = 0;buttons_pressed < lvl;){ 
    buttonStateR = digitalRead(BUTTON_R);
    buttonStateG = digitalRead(BUTTON_G);
    buttonStateV = digitalRead(BUTTON_V);
    buttonStateA = digitalRead(BUTTON_A);
    
    if(buttonStateR == HIGH){
       play_note(buttons_pressed);
       digitalWrite(LED_R, HIGH);
       delay(300);
       noTone(BUZZER);
       digitalWrite(LED_R, LOW);
      leds_player[buttons_pressed] = LED_R;
      buttons_pressed ++;
    }
    if(buttonStateG == HIGH){
       play_note(buttons_pressed);
       digitalWrite(LED_G, HIGH);
       delay(300);
       noTone(BUZZER);
      digitalWrite(LED_G, LOW);
      leds_player[buttons_pressed] = LED_G;
      buttons_pressed ++;
    }
    if(buttonStateA == HIGH){
      play_note(buttons_pressed);
      digitalWrite(LED_A, HIGH);
      delay(300);
      noTone(BUZZER);
      digitalWrite(LED_A, LOW);
      leds_player[buttons_pressed] = LED_A;
      buttons_pressed ++;
    }
    if(buttonStateV == HIGH){
      play_note(buttons_pressed);
      digitalWrite(LED_V, HIGH);
      delay(300);
      noTone(BUZZER);
      digitalWrite(LED_V, LOW);
      leds_player[buttons_pressed] = LED_V;
      buttons_pressed ++;
    } 
  }
  bool win = true;
  for(int i = 0; i < lvl; i++){
    if(random_leds[i] != leds_player[i]){
      win = false;
      break;
    } 
  }

  if(win) {
     win_song();
     lvl_simon++;
     s_lvl++;
     simon_score ++;
  } 
  else {
    lose_song();
    simon_score --;
  }  
}

///WHACK 
void wait_whack(){
  int leds_player[w_lvl];
  int buttons_pressed = 0;
  bool win = true;
  int i = 0;
  for(buttons_pressed = 0; buttons_pressed < w_lvl;){

    if(i >= w_lvl){
      break;}
     //se aprinde un led random
    random_leds[i] = leds[random(0,3)];
    digitalWrite(random_leds[i], HIGH);
    tone(BUZZER,  melody[i], 1000 / noteDurations[i]);
    delay(500);
    noTone(BUZZER);
    digitalWrite(random_leds[i], LOW);
    unsigned long time_entered = millis();
    i++;
    
    //se asteapta sa se apase un buton, timpul de asteptare scade cu cat 
    //se inainteaza in nivel
    bool p = false;
    while((millis() - time_entered) < (1000 - i * 10)){
      buttonStateR = digitalRead(BUTTON_R);
      buttonStateG = digitalRead(BUTTON_G);
      buttonStateV = digitalRead(BUTTON_V);
      buttonStateA = digitalRead(BUTTON_A);
       if(buttonStateR == HIGH){
        p = true;
         play_note(buttons_pressed);
         digitalWrite(LED_R, HIGH);
         delay(300);
         noTone(BUZZER);
         digitalWrite(LED_R, LOW);
        leds_player[buttons_pressed] = LED_R;
        buttons_pressed ++;
      }
      if(buttonStateG == HIGH){
        p = true;
         play_note(buttons_pressed);
         digitalWrite(LED_G, HIGH);
         delay(300);
         noTone(BUZZER);
        digitalWrite(LED_G, LOW);
        leds_player[buttons_pressed] = LED_G;
        buttons_pressed ++;
      }
      if(buttonStateA == HIGH){
         p = true;
        play_note(buttons_pressed);
        digitalWrite(LED_A, HIGH);
        delay(300);
        noTone(BUZZER);
        digitalWrite(LED_A, LOW);
        leds_player[buttons_pressed] = LED_A;
        buttons_pressed ++;
      }
      if(buttonStateV == HIGH){
        p = true;
        play_note(buttons_pressed);
        digitalWrite(LED_V, HIGH);
        delay(300);
        noTone(BUZZER);
        digitalWrite(LED_V, LOW);
        leds_player[buttons_pressed] = LED_V;
        buttons_pressed ++;
      } 
    }

    //daca a trecut timpul si nu a apasat a pierdut
    if(p == false){
      win = false;
      break;
    } else {
      //daca a apasat si e gresit a pierdut 
      if(random_leds[i-1] != leds_player[buttons_pressed -1]){
        win = false;
        break;
        }
    }
       
  }

  
  if(!win) {
    lose_song();
    whack_score --;
    return ;
  }
 
  for(int i = 0; i < w_lvl; i++){
    if(random_leds[i] != leds_player[i]){
      win = false;
      break;
    } 
  }

  if(win) {
     win_song();
     lvl_whack++;
     w_lvl++;
     whack_score ++;
  } 
  else {
    lose_song();
    whack_score --;
  }

}

void play_whack(){
  if(lvl_whack == WHACKMAX){
     close_whack = 1;
     return ;
   }
   lcd.clear();
   lcd.print("WHACK - LVL");
   lcd.print(lvl_whack);
   lcd.setCursor(0,1);
   lcd.print("Your score: ");
   lcd.print(whack_score);
   delay(300);
   wait_whack(); 
}


void choose_game(){
  //setați cursorul pe prima linie, prima coloana
  lcd.setCursor(0,0);
  lcd.print("Choose the game");
  lcd.setCursor(0,1);
  lcd.print("L:Simon R:Whack");
   buttonStateR = digitalRead(BUTTON_R);
   buttonStateG = digitalRead(BUTTON_G);

   if(buttonStateR){
    digitalWrite(LED_R, HIGH);
    delay(300);
    digitalWrite(LED_R, LOW);
    lcd.clear();
    lcd.print("Your choice:");
    lcd.setCursor(0,1);
    lcd.print("Simon Says!");
    game_choosed = 1;
    
    delay(2000);
   }

   if(buttonStateG){
    digitalWrite(LED_G, HIGH);
    delay(300);
    digitalWrite(LED_G, LOW);
    lcd.clear();
    lcd.print("Your choice:");
    lcd.setCursor(0,1);
    lcd.print("Whack a mole!");
    game_choosed = 2;
    delay(2000);
   } 
}


void loop()
{
  if(close_simon) {
     lcd.clear();
     lcd.print("Game finished");
     lcd.setCursor(0,1);
     if(simon_score <= 1){
       lcd.print("Next, do better!");
      } else {
      lcd.print("Congratulation");
      } 
     delay(2000);
     game_started = 0;
     game_choosed = 0;
     close_simon = 0;
    //to do reset values for simon 
     lvl_simon = 0;
     simon_score = 0;
     s_lvl = 1; 
   }

   if(close_whack) {
     lcd.clear();
     lcd.print("Game finished");
     lcd.setCursor(0,1);
     if(whack_score <= 1){
       lcd.print("Next, do better!");
      } else {
      lcd.print("Congratulation");
      } 
     delay(2000);
     game_started = 0;
     game_choosed = 0;
     close_whack = 0;

     //reset the value 
     lvl_whack = 0;
     whack_score = 0;
     w_lvl = 1;
   }
   
  choose_game();
  //simon 
  if(game_choosed == 1 && game_started == 0){
     lcd.clear();
     lcd.print("Let the game");
     lcd.setCursor(0,1);
     lcd.print("begin!");
     delay(2000);
     game_started = 1;
      
    }
  //whack
  if(game_choosed == 2 && game_started == 0){
    
     lcd.clear();
     lcd.print("Let the game");
     lcd.setCursor(0,1);
     lcd.print("begin!");
     delay(2000);
     game_started = 1;

    }
    
   if(game_choosed == 1 && game_started == 1){
    play_simon();
   }

   if(game_choosed == 2 && game_started == 1){
    play_whack();
   }
  
  
}
